/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.message;

import java.util.Iterator;

/**
 * Extends the Message interface and allows sub messages to be retrieved using a
 * name. Note that it is optional to associate sub messages with names.
 * <code>MessageNC</code> is a <i>naming context </i> for its sub messages
 * having a name. This means that a message can not have two sub messages with
 * the same name.
 */
public interface MessageNC extends Message
{

  /**
   * Returns a sub message of this message, or <code>null</code> if this
   * message doesn't have a sub message with the specified name.
   * 
   * @param name the name of the sub message, as specified in the message type.
   * @return a sub message of this message.
   */
  Message getSubMessage(String name);

  /**
   * Returns an <code>Iterator</code> that iterate over the unnamed sub
   * messages of this message.
   * 
   * @return an iterator on the sub messages.
   */
  Iterator getUnnamedSubMessageIterator();

  /**
   * Returns an (eventually empty) array containing the unnamed sub messages of
   * this message.
   * 
   * @return an array containing the sub messages of this message.
   */
  Message[] getUnnamedSubMessages();

}