/**
 */

package org.objectweb.util.monolog.wrapper.javaLog;

import org.objectweb.util.monolog.api.Handler;

import java.io.UnsupportedEncodingException;
import java.util.logging.Filter;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;
import java.util.logging.FileHandler;
import java.util.logging.ConsoleHandler;
import java.util.Map;
import java.util.HashMap;

public class GenericHandler
        extends java.util.logging.Handler
        implements Handler {

    public java.util.logging.Handler handler = null;
    private boolean enable = true;
    protected byte type;
    protected String name;
    Map attributes = null;


    public GenericHandler(String name, byte type) {
        this.type = type;
        this.name = name;
        attributes = new HashMap();
    }

    public GenericHandler(String name, java.util.logging.Handler h, byte type) {
        handler = h;
        this.type = type;
        this.name = name;
        attributes = new HashMap();
    }

    public GenericHandler(String name, java.util.logging.Handler h) {
        handler = h;
        this.name = name;
        attributes = new HashMap();
        if (h instanceof FileHandler) {
            type = Handler.FILE_HANDLER_TYPE;
        } else if (h instanceof ConsoleHandler) {
            type = Handler.CONSOLE_HANDLER_TYPE;
        }
    }


    /**
     * It retrieves the name of the handler
     */
    public String getName() {
        return name;
    }

    /**
     * It assigns the name of the handler
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * It retrieves the Handler type
     */
    public byte getType() {
        return type;
    }

    /**
     * It retrieves the attributes of the handler
     */
    public String[] getAttributeNames() {
        return (String[]) attributes.keySet().toArray(new String[0]);
    }

    /**
     * It retrieves the value of an attribute value of the handler.
     * @param name is an attribute name
     */
    public Object getAttribute(String name) {
        return attributes.get(name);
    }

    /**
     * It assigns an attributte to the handler.
     * @param name is the attribute name
     * @param value is the attribute value
     * @return the old value is the attribute was already defined
     */
    public Object setAttribute(String name, Object value) {
        if (name.equalsIgnoreCase(Handler.PATTERN_ATTRIBUTE)) {
            handler.setFormatter(new MonologFormatter((String) value));
        }
        return attributes.put(name, value);
    }


    // OVERRIDE THE java.util.logging.Handler CLASS //
    //----------------------------------------------//

    /**
     * Close the Handler and free all associated resources.
     */
    public void close() {
        handler.close();
    }

    /**
     * Flush any buffered output.
     */
    public void flush() {
        handler.flush();
    }

    /**
     * Return the character encoding for this Handler.
     */
    public String getEncoding() {
        return handler.getEncoding();
    }

    /**
     * Get the current Filter for this Handler.
     */
    public Filter getFilter() {
        return handler.getFilter();
    }

    /**
     * Return the Formatter for this Handler.
     */
    public Formatter getFormatter() {
        return handler.getFormatter();
    }

    /**
     * Get the log level specifying which messages will be logged by this Handler.
     */
    public java.util.logging.Level getLevel() {
        System.out.println("handler("+ name + ").getLevel(): " + handler.getLevel().getName());
        return handler.getLevel();
    }

    /**
     * Check if this Handler would actually log a given LogRecord.
     */
    public boolean isLoggable(LogRecord record) {
        //System.out.println("handler("+ name + ").isLoggable(" + record.getLevel().getName() + "): " + handler.isLoggable(record));
        return handler.isLoggable(record);
    }

    /**
     * Publish a LogRecord.
     */
    public void publish(LogRecord record) {
        //System.out.println("handler("+ name + ").publish(" + record.getLevel().getName() + "): " + handler.isLoggable(record));
        handler.publish(record);
    }

    /**
     * Set the character encoding used by this Handler.
     */
    public void setEncoding(String encoding)
            throws SecurityException, UnsupportedEncodingException {
        handler.setEncoding(encoding);
    }

    /**
     * Set the most recent IO exception.
     */
    protected void setException(Exception exception) {
    }

    /**
     * Set a Filter to control output on this Handler.
     */
    public void setFilter(Filter newFilter) {
        handler.setFilter(newFilter);
    }

    /**
     * Set a Formatter.
     */
    public void setFormatter(Formatter newFormatter) {
        handler.setFormatter(newFormatter);
    }

    /**
     * Set the log level specifying which message levels will
     * be logged by this Handler.
     */
    public void setLevel(java.util.logging.Level newLevel) {
        handler.setLevel(newLevel);
    }

}
